#!/bin/bash
#
# Connections manager script
#

LOCK_FILE=/tmp/connection.changing

[ -e "/bin/systemctl" ] && SYSTEMD=1

get_connections()
{
    #. /home/madevel/proj/gma/gma3/resource/software/test.sh
    #DEV[1]=enp6s0 HWADDR[1]=E4:4F:29:00:90:41 CARRIER[1]=on CONNECTION[1]="Con3" IPV4_COUNT[1]=1 IPV4[1]=10.0.0.43/16 IPV4_GW[1]=10.0.255.254 IPV4_DHCPLL_ADDRESS[1]=-- IPV6_COUNT[1]=1 IPV6[1]=fe80::7c95:e6d8:afe:458/64 IPV6_GW[1]=-- DEV[2]=enp2s0 HWADDR[2]=E4:4F:29:00:90:43 CARRIER[2]=on CONNECTION[2]="Con1" IPV4_COUNT[2]=1 IPV4[2]=172.16.61.21/16 IPV4_GW[2]=-- IPV4_DHCPLL_ADDRESS[2]=-- IPV6_COUNT[2]=1 IPV6[2]=fe80::9c47:b5d3:ec58:eb80/64 IPV6_GW[2]=-- DEV[3]=enp7s0 HWADDR[3]=E4:4F:29:00:90:40 CARRIER[3]=on CONNECTION[3]="imx6" IPV4_COUNT[3]=1 IPV4[3]=192.168.33.1/24 IPV4_GW[3]=-- IPV4_DHCPLL_ADDRESS[3]=-- IPV6_COUNT[3]=1 IPV6[3]=fe80::874:20c2:456d:6d6a/64 IPV6_GW[3]=-- DEV[4]=enp3s0 HWADDR[4]=E4:4F:29:00:90:42 CARRIER[4]=off CONNECTION[4]="--" IPV4_COUNT[4]=-- IPV4[4]=-- IPV4_GW[4]=-- IPV4_DHCPLL_ADDRESS[4]=-- IPV6_COUNT[4]=-- IPV6[4]=-- IPV6_GW[4]=--

    if [ -f "$LOCK_FILE" ]
    then
        echo "Connection change is already running. Skipped." >> /tmp/get_connection.log
        exit 1
    fi

    let CON_COUNT=0
    for con_name in $(nmcli -t connection show | cut -f1 -d':' | grep -v "ll")
    do
        eval $(awk -F $'\t' -v i="$CON_COUNT" 'BEGIN{
            delete if_name;
            delete con_name;
            }
            {
            if ($1 == "connection.interface-name") if_name[i]=$2;
            else if ($1 == "connection.id") con_name[i]=$2;
            }
            END{
                printf " INTERFACE_NAME[%d]=%s CONNECTION_NAME[%d]=%s ",i,if_name[i],i,con_name[i];
            }' < <(nmcli -t -f connection.interface-name,connection.id connection show "$con_name" | sed 's/:/\t/') )
            let CON_COUNT+=1
    done

#WIRED-PROPERTIES.CARRIER
#on/off

#GENERAL.STATE
#20 (unavailable)
#70 (connecting (getting IP configuration))
#100 (connected)

    eval $(awk -F $'\t' 'BEGIN{
    count=0;
    delete device;
    delete type;
    delete hwaddr;
    delete carrier;
    delete state;
    delete connection;
    delete ipv4;
    delete ipv4_count;
    delete ipv4_gw;
    delete ipv4_dhcpll_address;
    delete ipv6;
    delete ipv6_count;
    delete ipv6_gw;

    delete ipv4_route_count;
    delete ipv4_route;
    delete ipv4_dns_count;
    delete ipv4_dns;
    delete ipv6_route_count;
    delete ipv6_route;
    delete ipv6_dns_count;
    delete ipv6_dns;
    }
    {
        if ($1 == "GENERAL.DEVICE") {count++; device[count]=$2;}
        else if ($1 == "GENERAL.TYPE") type[count]=$2;
        else if ($1 == "GENERAL.HWADDR") hwaddr[count]=$2;
        else if ($1 == "WIRED-PROPERTIES.CARRIER") carrier[count]=$2;
        else if ($1 == "GENERAL.STATE") state[count]=$2;
        else if ($1 == "GENERAL.CONNECTION") connection[count]=$2;
        else if (substr($1,1,12) == "IP4.ADDRESS[")
        {
            if (match($1, /\[[0-9]+\]/))
            {
                ipv4_count[count]=substr($1,RSTART+1,RLENGTH-2);
                if (ipv4_count[count] == 1)
                {
                    ipv4[count]=$2;
                }
                else ipv4[count]=ipv4[count] "," $2;
                if (($2 ~ /^169\.254\..*/) && (substr(connection[count],5,2) == "ll"))
                {
                    sub(/\/.*/, "", $2);
                    ipv4_dhcpll_address[count]=$2;
                }
            }
        }
        else if ($1 == "IP4.GATEWAY") ipv4_gw[count]=$2;
        else if (substr($1,1,12) == "IP6.ADDRESS[")
        {
            if (match($1, /\[[0-9]+\]/))
            {
                ipv6_count[count]=substr($1,RSTART+1,RLENGTH-2);
                if (ipv6_count[count] == 1) ipv6[count]=$2;
                else ipv6[count]=ipv6[count] "," $2;
            }
        }
        else if ($1 == "IP6.GATEWAY") ipv6_gw[count]=$2;

        else if (substr($1,1,10) == "IP4.ROUTE[")
        {
            if (match($1, /\[[0-9]+\]/))
            {
                ipv4_route_count[count]=substr($1,RSTART+1,RLENGTH-2);
                gsub(/ /, "", $2)
                if (ipv4_route_count[count] == 1)
                {
                    ipv4_route[count]=$2;
                }
                else ipv4_route[count]=ipv4_route[count] "," $2;
            }
        }
        else if (substr($1,1,8) == "IP4.DNS[")
        {
            if (match($1, /\[[0-9]+\]/))
            {
                ipv4_dns_count[count]=substr($1,RSTART+1,RLENGTH-2);
                if (ipv4_dns_count[count] == 1)
                {
                    ipv4_dns[count]=$2;
                }
                else ipv4_dns[count]=ipv4_dns[count] "," $2;
            }
        }
        else if (substr($1,1,10) == "IP6.ROUTE[")
        {
            if (match($1, /\[[0-9]+\]/))
            {
                ipv6_route_count[count]=substr($1,RSTART+1,RLENGTH-2);
                gsub(/ /, "", $2)
                if (ipv6_route_count[count] == 1)
                {
                    ipv6_route[count]=$2;
                }
                else ipv6_route[count]=ipv6_route[count] "," $2;
            }
        }
        else if (substr($1,1,8) == "IP6.DNS[")
        {
            if (match($1, /\[[0-9]+\]/))
            {
                ipv6_dns_count[count]=substr($1,RSTART+1,RLENGTH-2);
                if (ipv6_dns_count[count] == 1)
                {
                    ipv6_dns[count]=$2;
                }
                else ipv6_dns[count]=ipv6_dns[count] "," $2;
            }
        }
    }
    END{
        for (i = 1; i <= count; i++)
        {
            if (type[i] == "ethernet")
            {
                printf " DEV[%d]=%s HWADDR[%d]=%s",i,device[i],i,hwaddr[i];

                if ((carrier[i] == "on") && ((state[i] == "100 (connected)") || (state[i] == "30 (disconnected)")))
                    printf " CARRIER[%d]=on",i;
                else
                    printf " CARRIER[%d]=off",i;

                if (i in connection && connection[i] != "")
                    printf " CONNECTION[%d]=\"%s\"",i,connection[i];
                else
                    printf " CONNECTION[%d]=\"--\"",i;

                if (i in ipv4_count && ipv4_count[i] != "")
                    printf " IPV4_COUNT[%d]=%s",i,ipv4_count[i];
                else
                    printf " IPV4_COUNT[%d]=1",i;

                if (i in ipv4 && ipv4[i] != "")
                    printf " IPV4[%d]=%s",i,ipv4[i];
                else
                    printf " IPV4[%d]=0.0.0.0/8",i;

                if (i in ipv4_gw && ipv4_gw[i] != "")
                    printf " IPV4_GW[%d]=%s",i,ipv4_gw[i];
                else
                    printf " IPV4_GW[%d]=--",i;

                if (i in ipv4_dhcpll_address && ipv4_dhcpll_address[i] != "")
                    printf " IPV4_DHCPLL_ADDRESS[%d]=%s",i,ipv4_dhcpll_address[i];
                else
                    printf " IPV4_DHCPLL_ADDRESS[%d]=--",i;


                if (i in ipv6_count && ipv6_count[i] != "")
                    printf " IPV6_COUNT[%d]=%s",i,ipv6_count[i];
                else
                    printf " IPV6_COUNT[%d]=1",i;

                if (i in ipv6 && ipv6[i] != "")
                    printf " IPV6[%d]=%s",i,ipv6[i];
                else
                    printf " IPV6[%d]=::/8",i;

                if (i in ipv6_gw && ipv6_gw[i] != "")
                    printf " IPV6_GW[%d]=%s",i,ipv6_gw[i];
                else
                    printf " IPV6_GW[%d]=--",i;


                if (i in ipv4_route_count && ipv4_route_count[i] != "")
                {
                    printf " IPV4_ROUTE_COUNT[%d]=%s",i,ipv4_route_count[i];
                    if (i in ipv4_route && ipv4_route[i] != "")
                        printf " IPV4_ROUTE[%d]=\"%s\"",i,ipv4_route[i];
                    else
                        printf " IPV4_ROUTE[%d]=--",i;
                }
                else
                    printf " IPV4_ROUTE_COUNT[%d]=0",i;


                if (i in ipv4_dns_count && ipv4_dns_count[i] != "")
                {
                    printf " IPV4_DNS_COUNT[%d]=%s",i,ipv4_dns_count[i];
                    if (i in ipv4_dns && ipv4_dns[i] != "")
                        printf " IPV4_DNS[%d]=%s",i,ipv4_dns[i];
                    else
                        printf " IPV4_DNS[%d]=--",i;
                }
                else
                    printf " IPV4_DNS_COUNT[%d]=0",i;

                if (i in ipv6_route_count && ipv6_route_count[i] != "")
                {
                    printf " IPV6_ROUTE_COUNT[%d]=%s",i,ipv6_route_count[i];
                    if (i in ipv6_route && ipv6_route[i] != "")
                        printf " IPV6_ROUTE[%d]=\"%s\"",i,ipv6_route[i];
                    else
                        printf " IPV6_ROUTE[%d]=--",i;
                }
                else
                    printf " IPV6_ROUTE_COUNT[%d]=0",i;

                if (i in ipv6_dns_count && ipv6_dns_count[i] != "")
                {
                    printf " IPV6_DNS_COUNT[%d]=%s",i,ipv6_dns_count[i];
                    if (i in ipv6_dns && ipv6_dns[i] != "")
                        printf " IPV6_DNS[%d]=%s",i,ipv6_dns[i];
                    else
                        printf " IPV6_DNS[%d]=--",i;
                }
                else
                    printf " IPV6_DNS_COUNT[%d]=0",i;

            }
        }
    }' < <(nmcli -t device show | sed 's/:/\t/') )

    for i in "${!CONNECTION[@]}"
    do
        CONNECTION[$i]="${CONNECTION[$i]%"ll"}"

        if [ "${CONNECTION[$i]}" == "--" ]
        then
            for ((ii=0;ii<$CON_COUNT;ii++))
            do
                if [ "${DEV[$i]}" == "${INTERFACE_NAME[$ii]}" ]
                then
                    CONNECTION[$i]=${CONNECTION_NAME[$ii]}
                    break
                fi
            done
        fi

        if [[ "${CONNECTION[$i]}" =~ imx6.* ]]
        then
            continue
        fi

        if [ "${CONNECTION[$i]}" != "--" ]
        then

            if [[ ! "$SYSTEMD" ]]
            then
                if [ -z "$(grep "${CARRIER[$i]}" "/tmp/${CONNECTION[$i]}.status" 2>/dev/null)" ]
                then
                    LOCK_FILE_CON=/tmp/${CONNECTION[$i]}.changing

                    if [ ! -f "$LOCK_FILE_CON" ]
                    then
                        echo $$ > "$LOCK_FILE_CON"

                        /etc/init.d/S46system-config updateinterfaces ${DEV[$i]} >/tmp/${CONNECTION[$i]}.log 2>/tmp/${CONNECTION[$i]}.err

                        echo "${CARRIER[$i]}" > "/tmp/${CONNECTION[$i]}.status"
                        rm -f "$LOCK_FILE_CON"
                    fi
                fi
            fi

            declare $(awk -F $'\t' -v i="$i" 'BEGIN{
            delete ipv4_method;
            delete ipv6_method;
            delete ipv4_dhcp_address;
            delete ipv6_dhcp_address;

            delete ipv4_dhcp_timeout;
            delete ipv6_dhcp_timeout;

            delete ipv4;
            delete ipv4_count;
            delete ipv4_gw;
            delete ipv6;
            delete ipv6_count;
            delete ipv6_gw;

            }
            {
            if ($1 == "ipv4.method") ipv4_method[i]=$2;
            else if ($1 == "ipv6.method") ipv6_method[i]=$2;
            else if ($1 == "ipv4.gateway") ipv4_gw[i]=$2;
            else if ($1 == "ipv6.gateway") ipv6_gw[i]=$2;
            else if ($1 == "ipv4.dhcp-timeout") ipv4_dhcp_timeout[i]=$2;
            else if ($1 == "ipv6.dhcp-timeout") ipv6_dhcp_timeout[i]=$2;

            else if (substr($1,1,12) == "DHCP4.OPTION") {
                    if (substr($2,1,13) == "ip_address = ") {
                        ipv4_dhcp_address[i]=substr($2,14);
                    }
                }
            else if (substr($1,1,12) == "DHCP6.OPTION") {
                    if (substr($2,1,13) == "ip_address = ") {
                        ipv6_dhcp_address[i]=substr($2,14);
                    }
                }
            else if ($1 == "ipv4.addresses")
            {
                ipv4_count[i]=gsub(/, /,",", $2)+1;
                ipv4[i]=$2;
            }
            else if ($1 == "ipv6.addresses")
            {
                ipv6_count[i]=gsub(/, /,",", $2)+1;
                ipv6[i]=$2;
            }
            }
            END{
                printf " IPV4_METHOD[%d]=%s IPV6_METHOD[%d]=%s",i,ipv4_method[i],i,ipv6_method[i];

                if (i in ipv4_dhcp_address && ipv4_dhcp_address[i] != "")
                    printf " IPV4_DHCP_ADDRESS[%d]=%s",i,ipv4_dhcp_address[i];
                else
                    printf " IPV4_DHCP_ADDRESS[%d]=--",i;

                if (i in ipv6_dhcp_address && ipv6_dhcp_address[i] != "")
                    printf " IPV6_DHCP_ADDRESS[%d]=%s",i,ipv6_dhcp_address[i];
                else
                    printf " IPV6_DHCP_ADDRESS[%d]=--",i;

                if (i in ipv4_count && ipv4_count[i] != "")
                    printf " IPV4_COUNT_OFF[%d]=%s",i,ipv4_count[i];
                else
                    printf " IPV4_COUNT_OFF[%d]=1",i;

                if (i in ipv4 && ipv4[i] != "")
                    printf " IPV4_OFF[%d]=%s",i,ipv4[i];
                else
                    printf " IPV4_OFF[%d]=0.0.0.0/8",i;

                if (i in ipv4_gw && ipv4_gw[i] != "")
                    printf " IPV4_GW_OFF[%d]=%s",i,ipv4_gw[i];
                else
                    printf " IPV4_GW_OFF[%d]=--",i;

                if (i in ipv6_count && ipv6_count[i] != "")
                    printf " IPV6_COUNT_OFF[%d]=%s",i,ipv6_count[i];
                else
                    printf " IPV6_COUNT_OFF[%d]=1",i;

                if (i in ipv6 && ipv6[i] != "")
                    printf " IPV6_OFF[%d]=%s",i,ipv6[i];
                else
                    printf " IPV6_OFF[%d]=::/8",i;

                if (i in ipv6_gw && ipv6_gw[i] != "")
                    printf " IPV6_GW_OFF[%d]=%s",i,ipv6_gw[i];
                else
                    printf " IPV6_GW_OFF[%d]=--",i;

                if (i in ipv4_dhcp_timeout && ipv4_dhcp_timeout[i] != "")
                    printf " IPV4_DHCP_TIMEOUT[%d]=%s",i,ipv4_dhcp_timeout[i];
                else
                    printf " IPV4_DHCP_TIMEOUT[%d]=--",i;

                if (i in ipv6_dhcp_timeout && ipv6_dhcp_timeout[i] != "")
                    printf " IPV6_DHCP_TIMEOUT[%d]=%s",i,ipv6_dhcp_timeout[i];
                else
                    printf " IPV6_DHCP_TIMEOUT[%d]=--",i;

            }' < <(nmcli -t -f ipv4.method,ipv4.addresses,ipv4.gateway,DHCP4,ipv6.method,ipv6.addresses,ipv6.gateway,DHCP6,ipv4.dhcp-timeout connection show "${CONNECTION[$i]}" | sed 's/:/\t/') )
            #,ipv6.dhcp-timeout #TODO: nmcli version 1.18 on i.MX6 has no ipv6.dhcp-timeout

            declare $(awk -v i="$i" '{
            if ($1 == "Speed:") {
                sub (/Mb\/s/,"",$2);
                sub (/Unknown!/,"0",$2);
                speed[i]=$2;
            }
            }
            END{
                printf " SPEED[%d]=%s",i,speed[i];
            }' < <(ethtool ${DEV[$i]} 2>/dev/null) )

            [ "${IPV4_DHCPLL_ADDRESS[$i]}" != "--" ] && IPV4_DHCP_ADDRESS[$i]=${IPV4_DHCPLL_ADDRESS[$i]}

            #echo -n "${CONNECTION[$i]},${DEV[$i]},${HWADDR[$i]},${SPEED[$i]},${CARRIER[$i]},${IPV4_METHOD[$i]},${IPV4_DHCP_ADDRESS[$i]},${IPV4_COUNT[$i]},${IPV4[$i]},${IPV4_GW[$i]},${IPV6_METHOD[$i]},${IPV6_DHCP_ADDRESS[$i]},${IPV6_COUNT[$i]},${IPV6[$i]},${IPV6_GW[$i]},
            #${IPV4_DHCP_TIMEOUT[$i]}, ### ${IPV6_DHCP_TIMEOUT[$i]},
            #${IPV4_ROUTE_COUNT[$i]},${IPV4_ROUTE[$i]},${IPV4_DNS_COUNT[$i]},${IPV4_DNS[$i]},${IPV6_ROUTE_COUNT[$i]},${IPV6_ROUTE[$i]},${IPV6_DNS_COUNT[$i]},${IPV6_DNS[$i]}"

            #################################

            #Sorting of IP addresses to <DHCP IPs>,<other IPs>,<ArtNet IPs>

            # Comma-separated string of IP addresses
            IP_STRING="${IPV4[$i]}"

            IPV4[$i]=$(echo "$IP_STRING" | tr ',' '\n' | awk -v dhcpip="${IPV4_DHCP_ADDRESS[$i]}" '{
                if ($0 ~ dhcpip) {
                    dhcp_ips = dhcp_ips $0 ","
                } else if ($0 ~ /^2\..*|^10\..*/) {
                    last_ips = last_ips $0 ","
                } else {
                    middle_ips = middle_ips $0 ","
                }
            } END {
                if (length(dhcp_ips) > 0) {
                    printf "%s", dhcp_ips
                }
                if (length(middle_ips) > 0) {
                    printf "%s", middle_ips
                }
                if (length(last_ips) > 0) {
                    printf "%s", last_ips
                }
            }')

            IPV4[$i]=${IPV4[$i]%,}

            #################################

            echo -n "${CONNECTION[$i]},${DEV[$i]},${HWADDR[$i]},${SPEED[$i]},${CARRIER[$i]},${IPV4_METHOD[$i]},${IPV4_DHCP_ADDRESS[$i]},"

            if [ "${IPV4_COUNT[$i]}" != "1" ]
            then
                echo -n "${IPV4_COUNT[$i]},${IPV4[$i]},"
            else
                if [ "${IPV4[$i]}" != "0.0.0.0/8" ]
                then
                    echo -n "${IPV4_COUNT[$i]},${IPV4[$i]},"
                else
                    echo -n "${IPV4_COUNT_OFF[$i]},${IPV4_OFF[$i]},"
                fi
            fi
            if [ "${IPV4_GW[$i]}" != "--" ]
            then
                echo -n "${IPV4_GW[$i]},"
            else
                echo -n "${IPV4_GW_OFF[$i]},"
            fi

            echo -n "${IPV6_METHOD[$i]},${IPV6_DHCP_ADDRESS[$i]},"

            if [ "${IPV6_COUNT[$i]}" != "1" ]
            then
                echo -n "${IPV6_COUNT[$i]},${IPV6[$i]},"
            else
                if [ "${IPV6[$i]}" != "::/8" ]
                then
                    echo -n "${IPV6_COUNT[$i]},${IPV6[$i]},"
                else
                    echo -n "${IPV6_COUNT_OFF[$i]},${IPV6_OFF[$i]},"
                fi
            fi
            if [ "${IPV6_GW[$i]}" != "--" ]
            then
                echo -n "${IPV6_GW[$i]},"
            else
                echo -n "${IPV6_GW_OFF[$i]},"
            fi

            echo -n "${IPV4_DHCP_TIMEOUT[$i]},"
            #${IPV6_DHCP_TIMEOUT[$i]}," #TODO: nmcli version 1.18 on i.MX6 has no ipv6.dhcp-timeout

            if [ "${IPV4_ROUTE_COUNT[$i]}" != "0" ]
            then
                echo -n "${IPV4_ROUTE_COUNT[$i]},${IPV4_ROUTE[$i]},"
            else
                echo -n "0,"
            fi
            if [ "${IPV4_DNS_COUNT[$i]}" != "0" ]
            then
                echo -n "${IPV4_DNS_COUNT[$i]},${IPV4_DNS[$i]},"
            else
                echo -n "0,"
            fi
            if [ "${IPV6_ROUTE_COUNT[$i]}" != "0" ]
            then
                echo -n "${IPV6_ROUTE_COUNT[$i]},${IPV6_ROUTE[$i]},"
            else
                echo -n "0,"
            fi
            if [ "${IPV6_DNS_COUNT[$i]}" != "0" ]
            then
                echo -n "${IPV6_DNS_COUNT[$i]},${IPV6_DNS[$i]}"
            else
                echo -n "0"
            fi



            echo ""

            #IPV4_ADDRESSES has only static IPs; IPV4_[123] have static *and* DHCP IPs. Now we should found DHCP address and print it at the last position

            #IPV4_ADDRESSES[$i]="${IPV4_ADDRESSES[$i]//-/x}"

            #echo "${IPV4_COUNT[$i]},${IPV4[$i]},${IPV4_GW[$i]},${IPV4_DHCP_ADDRESS[$i]},${IPV6_METHOD[$i]},${IPV6_COUNT[$i]},${IPV6[$i]},${IPV6_GW[$i]},${IPV6_DHCP_ADDRESS[$i]}"

            #if $(echo "${IPV4_ADDRESSES[$i]}" | grep -q -v -- "${IPV4_1[$i]}")
            #then
            #    echo -n "${IPV4_2[$i]},${IPV4_3[$i]},${IPV4_1[$i]},${IPV4_GW[$i]},${IPV6_METHOD[$i]},"
            #elif $(echo "${IPV4_ADDRESSES[$i]}" | grep -q -v -- "${IPV4_2[$i]}")
            #then
            #    echo -n "${IPV4_1[$i]},${IPV4_3[$i]},${IPV4_2[$i]},${IPV4_GW[$i]},${IPV6_METHOD[$i]},"
            #elif $(echo "${IPV4_ADDRESSES[$i]}" | grep -q -v -- "${IPV4_3[$i]}")
            #then
            #    echo -n "${IPV4_1[$i]},${IPV4_2[$i]},${IPV4_3[$i]},${IPV4_GW[$i]},${IPV6_METHOD[$i]},"
            #fi

            #IPV6_ADDRESSES[$i]="${IPV6_ADDRESSES[$i]//-/x}"

            #if $(echo "${IPV6_ADDRESSES[$i]}" | grep -q -v -- "${IPV6_1[$i]}")
            #then
            #    echo "${IPV6_2[$i]},${IPV6_3[$i]},${IPV6_1[$i]},${IPV6_GW[$i]}"
            #elif $(echo "${IPV6_ADDRESSES[$i]}" | grep -q -v -- "${IPV6_2[$i]}")
            #then
            #    echo "${IPV6_1[$i]},${IPV6_3[$i]},${IPV6_2[$i]},${IPV6_GW[$i]}"
            #elif $(echo "${IPV6_ADDRESSES[$i]}" | grep -q -v -- "${IPV6_3[$i]}")
            #then
            #    echo "${IPV6_1[$i]},${IPV6_2[$i]},${IPV6_3[$i]},${IPV6_GW[$i]}"
            #fi
        fi
    done
}
change_settings()
{
    CON=$1
    IF=$2
    SETTINGS=$3

    NET_PROGRESS=0

    if [ -f "$LOCK_FILE" ]
    then
        echo "Connection change is already running"
    else
        echo $$ > "$LOCK_FILE"

        nmcli device disconnect $IF

        CMD="nmcli connection modify \"$CON\" $SETTINGS"
        echo "$CMD"
        eval $CMD

        #CMD="nmcli connection modify \"$CONll\" $SETTINGS"
        #echo "$CMD"
        #eval $CMD

        #nmcli device connect $IF
        [ "$CON" != "Con3" ] && nmcli -w 15 connection up "$CON"ll
        nmcli -w 15 connection up "$CON"
        RC=$?

        if [[ ! "$SYSTEMD" ]]
        then
            if [ "$RC" != "0" ]
            then
                echo "Activating link-local connection..."
                nmcli connection up $CON"ll" #there is no link on this interface
            fi

            while [ -z "$(ifconfig $IF | grep "inet addr:")" ] && [ "$NET_PROGRESS" -lt 12 ]
            do
                echo -n " $NET_PROGRESS"
                let "NET_PROGRESS=$NET_PROGRESS+1"
                if [ "$NET_PROGRESS" -eq 10 ]
                then
                    echo -n "Activating link-local connection..."
                    nmcli connection up $CON"ll"
                fi
                sleep 1
            done
        fi
        rm -f "$LOCK_FILE"
    fi
}

case "$1" in
    get)
        get_connections
        ;;
    set)
        change_settings "$2" $3 "$4"
        ;;
    *)
        echo "Usage: $0 {get|set <Connection> <Interface> \"<Settings>\"}"
        echo "connections.sh set Con2 enp3s0 \"ipv4.method manual ipv4.gateway '' ipv4.addresses '2.3.3.3/8,2.4.4.4/8'\""
        exit 1
esac

exit $?

